package phase1;

public class SharingCar {

	/*
	mergeAlternateRequests that takes two LstRequest objects as
	parameters and returns a LstRequest object that contains
	elements from the two input lists alternatively. For example:
	{A,C,D,E,F,G} {B,J} -> {A,B,C,J,D,E,F,G}.
	*/
	
	public static LstRequest mergeAlternateRequests(LstRequest lst1, LstRequest lst2) {
		int size1=(lst1==null)?0:lst1.getSize();
		int size2=(lst2==null)?0:lst2.getSize();
		
		LstRequest lst=new LstRequest();
		
		int i1=0; 
		int i2=0; 
		
		while (i1<size1 && i2<size2) {
			lst.addLast(lst1.getAt(i1));
			i1++;
			lst.addLast(lst2.getAt(i2));
			i2++;
		}
		
		while (i1<size1) {
			lst.addLast(lst1.getAt(i1));
			i1++;
		}
		while (i2<size2) {
			lst.addLast(lst2.getAt(i2));
			i2++;
		}
		
		return lst;
	}
	
	/*
	 * sharing that receives two LstRequest objects as parameters and
returns a LstRequest object containing all the requests of both
input lists, whose origin cities are the same, and also their
destination cities. For example, if A is a request in the first list
whose origin city is Madrid and destination city Barcelona, and
B is a request in the second list whose origin city is also Madrid
and destination city Barcelona, then, the resulting list should
contain A and B requests.
	 */
	
	public static LstRequest sharing(LstRequest lst1, LstRequest lst2) {
		int size1=(lst1==null)?0:lst1.getSize();
		int size2=(lst2==null)?0:lst2.getSize();
		
		LstRequest lst=new LstRequest();
		
		for (int i=0; i<size1; i++) {
			Request r1=lst1.getAt(i);
			for (int j=0; j<size2; j++) {
				Request r2=lst2.getAt(j);
				if (r1.origin.equals(r2.origin) && r1.destination.equals(r2.destination)) {
					lst.addLast(r1);
					lst.addLast(r2);
				}
			}
		}
		
		return lst;
	}
	
	
    
	/*
	 * sort that takes a LstRequest object and an integer. The method
must return a LstRequest object containing the requests of the
input list, sorted in (ascending) alphabetical order by origin city
attribute (if the value of the integer parameter is 1), in
ascending alphabetical order by destination city (if the value is
2), and otherwise, in ascending alphabetical order by username.
	 */
	public static LstRequest sort(LstRequest lst, int opc) {
		int size=(lst==null)?0:lst.getSize();
		LstRequest sortedList=new LstRequest();
		for (int i=0; i<size;i++) {
			Request current=lst.getAt(i);
			//System.out.println(current.toString());
			sortedList.sortedInsert(current, opc);
		}
		return sortedList;
	}
	
	/*
	 * removeDuplicates that takes a LstRequest object and removes
all duplicated requests (that is, they share the same username,
origin and destination cities). The method must return a new list
without duplicated requests
	 */
	public static LstRequest removeDuplicates(LstRequest lst) {
		int size=(lst==null)?0:lst.getSize();
		LstRequest lstnew=new LstRequest();
		for (int i=0; i<size;i++) {
			Request current=lst.getAt(i);
			if (!lstnew.contains(current)) 
				lstnew.addLast(current);
		}
		return lstnew;
	}
	
	/*
	 * searchOrigin that takes a String parameter and a LstRequest
object as parameters, and returns a LstRequest object
containing all the requests whose origin city is the same as the
input string.
	 */
	public static LstRequest searchOrigin(String city,LstRequest lst) {
		int size=(lst==null)?0:lst.getSize();
		LstRequest lstnew=new LstRequest();
		for (int i=0; i<size;i++) {
			Request current=lst.getAt(i);
			String origin=current.origin;
			if (origin!=null && origin.equals(city))
				lstnew.addLast(current);
		}
		return lstnew;

	}
	
	public static LstRequest searchDestination(String city,LstRequest lst) {
		int size=(lst==null)?0:lst.getSize();
		LstRequest lstnew=new LstRequest();
		for (int i=0; i<size;i++) {
			Request current=lst.getAt(i);
			String origin=current.destination;
			if (origin!=null && origin.equals(city))
				lstnew.addLast(current);
		}
		return lstnew;

	}
	
	public static void main(String args[]) {
		LstRequest lst1=new LstRequest();
		LstRequest lst2=new LstRequest();

//		for (int i=1; i<=30; i++)
//			lst1.addLast(Request.createRandomRequest());
//		
//		LstRequests lst2=new LstRequests();
//		for (int i=1; i<=30; i++)
//			lst2.addLast(Request.createRandomRequest());
//		
//		System.out.println(lst1.toString());
//		System.out.println(lst2.toString());
//		LstRequests lst=SharingCar.mergeAlternateRequests(lst1, lst2);
//		System.out.println("merge:");
//		System.out.println(lst.toString());
////		lst=SharingCar.sharing(lst1, lst2);
////		System.out.println("sharing:");
////		System.out.println(lst.toString());
//		
////		lst=SharingCar.sort(lst, 1);
////		System.out.println("sort by origin:");
////		System.out.println(lst.toString());
//
////		lst=SharingCar.sort(lst, 2);
////		System.out.println("sort by destination:");
////		System.out.println(lst.toString());
//		
//		lst=SharingCar.sort(lst, 1);
//		System.out.println("sort by username:");
//		System.out.println(lst.toString());
//
//		lst=SharingCar.removeDuplicates(lst);
//				
//		System.out.println("removeDuplicates");
//		System.out.println(lst.toString());
//		
////		lst=SharingCar.searchOrigin("Madrid", lst);
////		System.out.println("searchOrigin Madrid");
////		System.out.println(lst.toString());
//		
//		lst=SharingCar.searchDestination("Sevilla", lst);
//		System.out.println("searchDestination Sevilla");
//		System.out.println(lst.toString());
		
		//input lists
				lst1.addLast(new Request("user1","Madrid","Barcelona"));
				lst1.addLast(new Request("user2","Alicante","Barcelona"));
				lst1.addLast(new Request("user3","Sevilla","Alicante"));
				lst1.addLast(new Request("user4","Valencia","Sevilla"));
				
				lst2.addLast(new Request("user5","Madrid","Sevilla"));
				lst2.addLast(new Request("user6","Madrid","Barcelona"));
				lst2.addLast(new Request("user7","Bilbao","Madrid"));
				lst2.addLast(new Request("user8","Valencia","Madrid"));
				lst2.addLast(new Request("user9","Valencia","Barcelona"));
				lst2.addLast(new Request("user6","Madrid","Barcelona"));
				lst2.addLast(new Request("user11","Sevilla","Madrid"));
				lst2.addLast(new Request("user12","Alicante","Barcelona"));
		
		LstRequest lst=SharingCar.sharing(lst1, lst2);
		System.out.println(lst.toString());

		
	}
}

