package phase2;

import java.util.Random;

public class Users implements IBSTree {

	BSTNode root;

	
	//search the node whose key is key and 
	//returns its associated value
	public User findUser(String username) {
		return findUser(root, username);
	}
	
	/**
	 * The method searches the key into the subtree 
	 * that hangs down from currentNode
	 * Note: It would be more correct 
	 * for the method to be in the BSTNode class
	 * @param currentNode
	 * @param key
	 * @return
	 */
	private static User findUser(BSTNode currentNode, String username) {
		User result=null;
		if (currentNode == null) {
			//System.out.prinltn(key + " does not exist!");
		} else {
			if (username.equals(currentNode.key))
				result= currentNode.user;
			else if (username.compareTo(currentNode.key) < 0)
				result=findUser(currentNode.left, username);
			else
				result=findUser(currentNode.right, username);
		}
		return result;
	}
	
	//creates a new node and inserts it into the tree
	//first, it must search its right position into the tree
	//the new nodes must always be inserted as leaves
	public void insertUser(User user) {
		BSTNode newNode=new BSTNode(user);
		if (root==null) root=newNode;
		else insert(newNode, root);
	}
	
	
	/**
	 * This method first looks for (recursively) the right position and then
	 * inserts the new node.
	 * Note: It would be more correct 
	 * for the method to be in the BSTNode class
	 * @param newNode 
	 * @param currentNode node where you start the search
	 * 
	 */
	protected static void insert(BSTNode newNode, BSTNode currentNode) {
		if (currentNode==null) return;
		String key=newNode.key;
		if (key.compareTo(currentNode.key)==0) {
			System.out.println(key + " already exists. Duplicates are not allowed!!!.");
			return;
		}
		if (key.compareTo(currentNode.key)<0) {
			if (currentNode.left==null) {
				//the method already finds the right position
				currentNode.left=newNode;
				newNode.parent=currentNode;
			} else {
				//the method must continue searching...
				insert(newNode,currentNode.left);
			}
		} else {
			if (currentNode.right==null) {
				//the method already finds the right position
				currentNode.right=newNode;
				newNode.parent=currentNode;
			} else {
				//the method must continue searching...
				insert(newNode,currentNode.right);
			}
		}
		
	}
	

	//searches the node whose key is key and removes it from the tree
	//the method uses a recursive method that returns the modified tree
	public void removeUser(String username) {
		root=remove(username,root);
	}

	/**
	 * 
	 * @param key
	 * @param currentNode
	 * @return
	 */
	private BSTNode remove(String key, BSTNode currentNode) {
		if (currentNode == null) {
			System.out.println("Cannot remove: The key doesn't exist");
			return null;
		}
		
		if (key.compareTo(currentNode.key)<0) {
			currentNode.left=remove(key,currentNode.left);
			return currentNode;
		}
		
		if (key.compareTo(currentNode.key)>0) {
			currentNode.right=remove(key,currentNode.right);
			return currentNode;

		}
		
		//Here, it means that key.compareTo(node.Key)==0
		//That is, we already have found it: node is the node to remove.
		//First case: the node is a leaf.
		if (currentNode.left==null && currentNode.right==null) {
			return null;
		}

		//Second case is one the node only has a child: left or right
		if (currentNode.left==null)  {
			currentNode.right.parent = currentNode.parent;
			return currentNode.right;
		}
		
		if (currentNode.right==null) {
			currentNode.left.parent = currentNode.parent;
			return currentNode.left;
		}
		
		//Third case: node has two childs.
		//We can replace its value by the maximum value in its left child or
		//by the minimum value in its right child

		BSTNode successorNode = currentNode.right;
		while (successorNode.left!=null) {
			successorNode=successorNode.left;
		}
		//replace its key and its elem by the successorNodes's key and elem
		currentNode.user=successorNode.user;
		currentNode.key=successorNode.key;
		
		//Finally, we must remove the sucesorNode (which is one of the two previous cases)
		currentNode.right=remove(successorNode.key,currentNode.right);
		return currentNode;

		 
		
	}

	public void complaint(String username) {
		User user=findUser(username);
		if (user!=null) { 
			user.num_complaints++;
			if (user.num_complaints==5) {
				removeUser(username);
			}
		}
	}
	/**
	 * returns an array
	 * array[0] contains the list of usernames with less complaints
	 * array[1] contains the list of usernames with more complaints
	 * @return
	 */
	public DList[] extremUsers() {
		DList[] arraylist=new DList[2];
		if (root==null) return arraylist;
		//user with less complaints
		arraylist[0]=new DList();
		int num=0; 
		while (arraylist[0].size==0 && num<5) {
			arraylist[0]=find_complaints(num);
			num++;
		}
		
		num=4;
		arraylist[1]=new DList();
		while (arraylist[1].size==0 && num>=0) {
			arraylist[1]=find_complaints(num);
			num--;
		}
		
		return arraylist;
	}	

	public DList find_complaints(int num) {
		DList list=new DList();
		return find_complaints(root,num,list);
	}	

	public static DList find_complaints(BSTNode node, int num, DList list) {
		if (node==null) return list;
		list=find_complaints(node.left,num,list);
		if (node.user.num_complaints==num) list.addLast(node.user.username);
		list=find_complaints(node.right,num,list);
		return list;
	}	

	
	public void show()  {
		 if (root!=null) root.showInOrder();
			else System.out.println("there are no users");
	}
	
	
		 
	public void showLevel() {
		if (root!=null) {
			root.showLevels();
		}
	}
	
	
//	public void showPreorder() {
//		if (root!=null) root.showPreOrder();
//		else System.out.println("tree is empty");
//	}
//
//	public void showInorder() {
//		 if (root!=null) root.showInOrder();
//		else System.out.println("tree is empty");
//	}
//	
//	public void showPostorder() {
//		if (root!=null) root.showPostOrder();
//		else System.out.println("tree is empty");
//	}
	
	
	public int getSize() {
		return BSTNode.getSize(root);
	}

	public int getHeight() {
		return BSTNode.getHeight(root);
	}

	
	public static boolean equals(BSTNode current, BSTNode obj) {
		if (current==null && obj==null) return true;
		if (obj==null) return false;
		
		return current.user.equals(obj.user) && equals(current.left,obj.left) 
				&& equals(current.right,obj.right);
	}
	
	public boolean equals(Users tree) {
		if (tree==null) return false;
		return equals(this.root,tree.root);
	}
	
	public  DList getSortedList() {
		DList list=new DList();
		return getSortedList(this.root,list);
	}
	
	public static DList getSortedList(BSTNode node, DList list)  {
		if (node != null) {
			list=getSortedList(node.left,list);
			list.addLast(node.key);
			list=getSortedList(node.right,list);
		}
		return list;
	}

	public static void main(String args[]) {
		Users tree=new Users();
		for (int i=1; i<=30; i++) {
			User obj=User.createRandomUser();
			System.out.println(obj.toString());
			tree.insertUser(obj);
		}
		//tree.show();
		//tree.showByLevel();
		tree.removeUser("zzd");
		tree.show();
		User user=tree.findUser("gazzd");
		if (user!=null) System.out.println(user.toString());
		tree.complaint("gazzd");
		user=tree.findUser("gazzd");
		while (user!=null) {
			System.out.println(user.toString());
			tree.complaint("gazzd");
			user=tree.findUser("gazzd");
		}
		
		Random rn=new Random();
		for (int i=1; i<=15; i++) {
			int index=rn.nextInt(User.USERS.length);
			String username=User.USERS[index];
			tree.complaint(username);
		}
		
		tree.show();
		DList[] list=tree.extremUsers();
		System.out.println("less complaints:" + list[0].toString());
		System.out.println("more complaints:" + list[1].toString());

	}
	
}